# Makefile for AFFLIB under Windows
#
# Remember:
# LIBC is single threaded
# LIBCMTD is multi-threaded and debug-enabled


OBJS = aff_db.obj \
	aff_toc.obj\
	afflib.obj \
	afflib_os.obj \
	afflib_pages.obj \
	afflib_stream.obj \
	afflib_util.obj \
	getopt_win32.obj \
	lzma_glue.obj \
	vnode_afd.obj \
	vnode_aff.obj \
	vnode_split_raw.obj \
	vnode_afm.obj \
	vnode_raw.obj 

INCS=/I ..\zlib123\include /I c:\OpenSSL\Include \
	/I ..\lzma443\c\7zip\Compress\LZMA_Alone

CC=cl

LIBS=ws2_32.lib\
	c:\openssl\lib\vc\ssleay32.lib \
        c:\openssl\lib\vc\libeay32.lib \
	..\zlib123\lib\zdll.lib

CFLAGSCOMMON = /Od /W3 /Yd /Gm /GX /Zi /ZI /GF /GZ /Ge \
	/D "WIN32" /D "_DEBUG" /D "_MBCS" /D "WIN32_NT"  \
	/D "MSC" /c /nologo 

CFLAGS = $(CFLAGSCOMMON) /Fp"slib.pch" /YX $(INCS) 


CPPFLAGS = $(CFLAGSCOMMON) $(INCS)

#
# C/C++ Compiler options
# Type CL /? for a current list
#
# -OPTIMIZATION-
# /Od       - disable optimizations
#
# -CODE GENERATION-
# /W4       - warning level 4
# /Yd       - put debug info in every .OBJ (rather than in vc60.pcb)
# /Gm       - enable minimal rebuild
# /GX       - enabled C++ EH
# /Zi       - enable debugging info
# /ZI       - enable full Edit and Continue info
# /GF       - enable read-only string pooling
# /GZ       - enable runtime debug checks
# /Ge       - force stack checking for all funcs
#
# -OUTPUT FILES-
# /Fp       - Precompiled headers
#
# -MISCELLANEOUS-
# /YX       - automatic .PCH
#
# -LINKING-
# /ML - Link with LIBC.LIB (singlethreaded)
# /MLd - Link with LIBCD.LIB (singlethreaded with debug)
# /MTd      - Link multithreaded with debug info

LINKSTD=/nologo /incremental:yes /pdb:"afflib.pdb" /map $(LIBS)
LINKDEBUG=$(LINKSTD) /debug

# Previously had /nodefaultlib:libcmt  in there

all: afflib.lib aftest.exe 

aftest.exe: afflib.lib aftest.obj
	link -out:aftest.exe aftest.obj afflib.lib $(LINKSTD) /nodefaultlib:libcmtd 

afflib.lib: $(OBJS) afflib.mak 
	-del afflib.lib
	lib /out:afflib.lib $(OBJS)

clean:
	-del *.obj
	-del *.exe
